@@ -1,29 +1,21 @@
-// Copyright 2021 Authors of Hubble
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Hubble
 
 package observe
 
 import (
 	"fmt"
+	"io"
 
+	"github.com/cilium/hubble/cmd/common/config"
 	hubprinter "github.com/cilium/hubble/pkg/printer"
 	hubtime "github.com/cilium/hubble/pkg/time"
 )
 
-func handleEventsArgs(debug bool) error {
+func handleEventsArgs(writer io.Writer, debug bool) error {
 	// initialize the printer with any options that were passed in
 	var opts = []hubprinter.Option{
+		hubprinter.Writer(writer),
 		hubprinter.WithTimeFormat(hubtime.FormatNameToLayout(formattingOpts.timeFormat)),
 	}
 
@@ -33,7 +25,11 @@ func handleEventsArgs(debug bool) error {
 	case "dict":
 		opts = append(opts, hubprinter.Dict())
 	case "json", "JSON":
-		opts = append(opts, hubprinter.JSON())
+		if config.Compat.LegacyJSONOutput {
+			opts = append(opts, hubprinter.JSONLegacy())
+			break
+		}
+		fallthrough
 	case "jsonpb":
 		opts = append(opts, hubprinter.JSONPB())
 	case "tab", "table":
