@@ -1,16 +1,5 @@
-// Copyright 2019-2021 Authors of Hubble
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Hubble
 
 package observe
 
@@ -25,10 +14,9 @@ import (
 	"os/signal"
 	"strings"
 	"text/tabwriter"
-	"time"
 
 	flowpb "github.com/cilium/cilium/api/v1/flow"
-	"github.com/cilium/cilium/api/v1/observer"
+	observerpb "github.com/cilium/cilium/api/v1/observer"
 	monitorAPI "github.com/cilium/cilium/pkg/monitor/api"
 	"github.com/cilium/hubble/cmd/common/config"
 	"github.com/cilium/hubble/cmd/common/conn"
@@ -42,8 +30,9 @@ import (
 	"github.com/spf13/viper"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
+	"google.golang.org/protobuf/types/known/fieldmaskpb"
 	"google.golang.org/protobuf/types/known/timestamppb"
-	"gopkg.in/yaml.v2"
+	"gopkg.in/yaml.v3"
 )
 
 // see protocol filter in Hubble server code (there is unfortunately no
@@ -51,6 +40,7 @@ import (
 var protocols = []string{
 	// L4
 	"icmp", "icmpv4", "icmpv6",
+	"sctp",
 	"tcp",
 	"udp",
 	// L7
@@ -65,18 +55,22 @@ var verdicts = []string{
 	flowpb.Verdict_AUDIT.String(),
 	flowpb.Verdict_REDIRECTED.String(),
 	flowpb.Verdict_ERROR.String(),
+	flowpb.Verdict_TRACED.String(),
+	flowpb.Verdict_TRANSLATED.String(),
 }
 
 // flowEventTypes are the valid event types supported by observe. This corresponds
 // to monitorAPI.MessageTypeNames, excluding MessageTypeNameAgent,
 // MessageTypeNameDebug and MessageTypeNameRecCapture. These excluded message
-// message types are not supported by `hubble observe flows` but have separate sub-commands.
+// types are not supported by `hubble observe flows` but have separate
+// sub-commands.
 var flowEventTypes = []string{
 	monitorAPI.MessageTypeNameCapture,
 	monitorAPI.MessageTypeNameDrop,
 	monitorAPI.MessageTypeNameL7,
 	monitorAPI.MessageTypeNamePolicyVerdict,
 	monitorAPI.MessageTypeNameTrace,
+	monitorAPI.MessageTypeNameTraceSock,
 }
 
 // flowEventTypeSubtypes is a map message types and all their subtypes.
@@ -99,6 +93,7 @@ var flowEventTypeSubtypes = map[string][]string{
 	},
 	monitorAPI.MessageTypeNameL7:            nil,
 	monitorAPI.MessageTypeNamePolicyVerdict: nil,
+	monitorAPI.MessageTypeNameTraceSock:     nil,
 }
 
 const (
@@ -114,7 +109,7 @@ type getFlowsFilters struct {
 
 // getFlowFiltersYAML returns allowlist/denylist filters as a YAML string. This YAML can then be
 // passed to hubble observe command via `--config` flag.
-func getFlowFiltersYAML(req *observer.GetFlowsRequest) (string, error) {
+func getFlowFiltersYAML(req *observerpb.GetFlowsRequest) (string, error) {
 	var allowlist, denylist []string
 	for _, filter := range req.Whitelist {
 		filterJSON, err := json.Marshal(filter)
@@ -142,9 +137,61 @@ func getFlowFiltersYAML(req *observer.GetFlowsRequest) (string, error) {
 	return string(out), nil
 }
 
-func newFlowsCmd(vp *viper.Viper, ofilter *flowFilter) *cobra.Command {
-	observeCmd := &cobra.Command{
-		Example: `* Piping flows to hubble observe
+// GetHubbleClientFunc is primarily used to mock out the hubble client in some unit tests.
+var GetHubbleClientFunc = func(ctx context.Context, vp *viper.Viper) (client observerpb.ObserverClient, cleanup func() error, err error) {
+	if otherOpts.inputFile != "" {
+		var f *os.File
+		if otherOpts.inputFile == "-" {
+			// read flows from stdin
+			f = os.Stdin
+			// noop cleanup
+			cleanup = func() error { return nil }
+		} else {
+			// read flows from the provided file
+			f, err = os.Open(otherOpts.inputFile)
+			if err != nil {
+				return nil, nil, err
+			}
+			cleanup = f.Close
+		}
+		client = NewIOReaderObserver(f)
+		return client, cleanup, nil
+	}
+	// read flows from a hubble server
+	hubbleConn, err := conn.New(ctx, vp.GetString(config.KeyServer), vp.GetDuration(config.KeyTimeout))
+	if err != nil {
+		return nil, nil, err
+	}
+	logger.Logger.WithField("server", config.KeyServer).Debug("connected to Hubble API")
+	cleanup = hubbleConn.Close
+	client = observerpb.NewObserverClient(hubbleConn)
+	return client, cleanup, nil
+}
+
+type cmdUsage struct {
+	use     string
+	short   string
+	long    string
+	example string
+}
+
+func newObserveCmd(vp *viper.Viper) *cobra.Command {
+	ofilter := newFlowFilter()
+	usage := cmdUsage{
+		use:   "observe",
+		short: "Observe flows and events of a Hubble server",
+	}
+	return newFlowsCmdHelper(usage, vp, ofilter)
+}
+
+func newFlowsCmd(vp *viper.Viper) *cobra.Command {
+	ofilter := newFlowFilter()
+	return newFlowsCmdWithFilter(vp, ofilter)
+}
+
+func newFlowsCmdWithFilter(vp *viper.Viper, ofilter *flowFilter) *cobra.Command {
+	usage := cmdUsage{
+		example: `* Piping flows to hubble observe
 
   Save output from 'hubble observe -o jsonpb' command to a file, and pipe it to
   the observe command later for offline processing. For example:
@@ -158,15 +205,26 @@ func newFlowsCmd(vp *viper.Viper, ofilter *flowFilter) *cobra.Command {
   Note that the observe command ignores --follow, --last, and server flags when it
   reads flows from stdin. The observe command processes and output flows in the same
   order they are read from stdin without sorting them by timestamp.`,
-		Use:   "observe",
-		Short: "Observe flows of a Hubble server",
-		Long: `Observe provides visibility into flow information on the network and
+		use:   "flows",
+		short: "Observe flows of a Hubble server",
+		long: `Observe provides visibility into flow information on the network and
 application level. Rich filtering enable observing specific flows related to
 individual pods, services, TCP connections, DNS queries, HTTP requests and
 more.`,
+	}
+
+	return newFlowsCmdHelper(usage, vp, ofilter)
+}
+
+func newFlowsCmdHelper(usage cmdUsage, vp *viper.Viper, ofilter *flowFilter) *cobra.Command {
+	flowsCmd := &cobra.Command{
+		Example: usage.example,
+		Use:     usage.use,
+		Short:   usage.short,
+		Long:    usage.long,
 		RunE: func(cmd *cobra.Command, args []string) error {
 			debug := vp.GetBool(config.KeyDebug)
-			if err := handleFlowArgs(ofilter, debug); err != nil {
+			if err := handleFlowArgs(cmd.OutOrStdout(), ofilter, debug); err != nil {
 				return err
 			}
 			req, err := getFlowsRequest(ofilter, vp.GetStringSlice(allowlistFlag), vp.GetStringSlice(denylistFlag))
@@ -185,24 +243,11 @@ more.`,
 			ctx, cancel := signal.NotifyContext(context.Background(), os.Interrupt, os.Kill)
 			defer cancel()
 
-			var client observer.ObserverClient
-			fi, err := os.Stdin.Stat()
+			client, cleanup, err := GetHubbleClientFunc(ctx, vp)
 			if err != nil {
 				return err
 			}
-			if fi.Mode()&os.ModeNamedPipe != 0 {
-				// read flows from stdin
-				client = newIOReaderObserver(os.Stdin)
-				logger.Logger.Debug("Reading flows from stdin")
-			} else {
-				// read flows from a hubble server
-				hubbleConn, err := conn.New(ctx, vp.GetString(config.KeyServer), vp.GetDuration(config.KeyTimeout))
-				if err != nil {
-					return err
-				}
-				defer hubbleConn.Close()
-				client = observer.NewObserverClient(hubbleConn)
-			}
+			defer cleanup()
 
 			logger.Logger.WithField("request", req).Debug("Sending GetFlows request")
 			if err := getFlows(ctx, client, req); err != nil {
@@ -217,67 +262,14 @@ more.`,
 		},
 	}
 
-	// selector flags
-	selectorFlags := pflag.NewFlagSet("selectors", pflag.ContinueOnError)
-	selectorFlags.BoolVar(&selectorOpts.all, "all", false, "Get all flows stored in Hubble's buffer")
-	selectorFlags.Uint64Var(&selectorOpts.last, "last", 0, fmt.Sprintf("Get last N flows stored in Hubble's buffer (default %d)", defaults.FlowPrintCount))
-	selectorFlags.Uint64Var(&selectorOpts.first, "first", 0, "Get first N flows stored in Hubble's buffer")
-	selectorFlags.BoolVarP(&selectorOpts.follow, "follow", "f", false, "Follow flows output")
-	selectorFlags.StringVar(&selectorOpts.since,
-		"since", "",
-		fmt.Sprintf(`Filter flows since a specific date. The format is relative (e.g. 3s, 4m, 1h43,, ...) or one of:
-  StampMilli:             %s
-  YearMonthDay:           %s
-  YearMonthDayHour:       %s
-  YearMonthDayHourMinute: %s
-  RFC3339:                %s
-  RFC3339Milli:           %s
-  RFC3339Micro:           %s
-  RFC3339Nano:            %s
-  RFC1123Z:               %s
- `,
-			time.StampMilli,
-			hubtime.YearMonthDay,
-			hubtime.YearMonthDayHour,
-			hubtime.YearMonthDayHourMinute,
-			time.RFC3339,
-			hubtime.RFC3339Milli,
-			hubtime.RFC3339Micro,
-			time.RFC3339Nano,
-			time.RFC1123Z,
-		),
-	)
-	selectorFlags.StringVar(&selectorOpts.until,
-		"until", "",
-		fmt.Sprintf(`Filter flows until a specific date. The format is relative (e.g. 3s, 4m, 1h43,, ...) or one of:
-  StampMilli:             %s
-  YearMonthDay:           %s
-  YearMonthDayHour:       %s
-  YearMonthDayHourMinute: %s
-  RFC3339:                %s
-  RFC3339Milli:           %s
-  RFC3339Micro:           %s
-  RFC3339Nano:            %s
-  RFC1123Z:               %s
- `,
-			time.StampMilli,
-			hubtime.YearMonthDay,
-			hubtime.YearMonthDayHour,
-			hubtime.YearMonthDayHourMinute,
-			time.RFC3339,
-			hubtime.RFC3339Milli,
-			hubtime.RFC3339Micro,
-			time.RFC3339Nano,
-			time.RFC1123Z,
-		),
-	)
-	observeCmd.PersistentFlags().AddFlagSet(selectorFlags)
-
 	// filter flags
 	filterFlags := pflag.NewFlagSet("filters", pflag.ContinueOnError)
 	filterFlags.Var(filterVar(
 		"not", ofilter,
 		"Reverses the next filter to be blacklist i.e. --not --from-ip 2.2.2.2"))
+	filterFlags.Var(filterVar(
+		"uuid", ofilter,
+		"Show the only flow matching this unique flow identifier, if any"))
 	filterFlags.Var(filterVar(
 		"node-name", ofilter,
 		`Show all flows which match the given node names (e.g. "k8s*", "test-cluster/*.company.com")`))
@@ -324,6 +316,10 @@ more.`,
 		"http-path", ofilter,
 		`Show only flows which match this HTTP path regular expressions (e.g. "/page/\\d+")`))
 
+	filterFlags.Var(filterVar(
+		"trace-id", ofilter,
+		"Show only flows which match this trace ID"))
+
 	filterFlags.Var(filterVar(
 		"from-fqdn", ofilter,
 		`Show all flows originating at the given fully qualified domain name (e.g. "*.cilium.io").`))
@@ -406,6 +402,16 @@ more.`,
 		"to-port", ofilter,
 		"Show only flows with the given destination port (e.g. 8080)"))
 
+	filterFlags.Var(filterVar(
+		"from-workload", ofilter,
+		"Show all flows originating at an endpoint with the given workload"))
+	filterFlags.Var(filterVar(
+		"workload", ofilter,
+		"Show all flows related to an endpoint with the given workload"))
+	filterFlags.Var(filterVar(
+		"to-workload", ofilter,
+		"Show all flows terminating at an endpoint with the given workload"))
+
 	filterFlags.Var(filterVar(
 		"from-identity", ofilter,
 		"Show all flows originating at an endpoint with the given security identity"))
@@ -415,90 +421,41 @@ more.`,
 	filterFlags.Var(filterVar(
 		"to-identity", ofilter,
 		"Show all flows terminating at an endpoint with the given security identity"))
-	observeCmd.Flags().AddFlagSet(filterFlags)
+	filterFlags.Var(filterVar(
+		"traffic-direction", ofilter,
+		"Show all flows in the given traffic direction (either ingress or egress)"))
 
 	rawFilterFlags := pflag.NewFlagSet("raw-filters", pflag.ContinueOnError)
 	rawFilterFlags.StringArray(allowlistFlag, []string{}, "Specify allowlist as JSON encoded FlowFilters")
 	rawFilterFlags.StringArray(denylistFlag, []string{}, "Specify denylist as JSON encoded FlowFilters")
-	observeCmd.Flags().AddFlagSet(rawFilterFlags)
 	// bind these flags to viper so that they can be specified as environment variables.
 	vp.BindPFlags(rawFilterFlags)
 
-	// formatting flags only to `hubble observe`, but not sub-commands. Will be added to
-	// generic formatting flags below.
-	observeFormattingFlags := pflag.NewFlagSet("", pflag.ContinueOnError)
-	observeFormattingFlags.BoolVar(
+	// formatting flags specific to flows
+	flowsFormattingFlags := pflag.NewFlagSet("Flow Format", pflag.ContinueOnError)
+	flowsFormattingFlags.BoolVar(
 		&formattingOpts.numeric,
 		"numeric",
 		false,
 		"Display all information in numeric form",
 	)
-	observeFormattingFlags.BoolVar(
+	flowsFormattingFlags.BoolVar(
 		&formattingOpts.enableIPTranslation,
 		"ip-translation",
 		true,
 		"Translate IP addresses to logical names such as pod name, FQDN, ...",
 	)
-	observeFormattingFlags.StringVar(
+	flowsFormattingFlags.StringVar(
 		&formattingOpts.color,
 		"color", "auto",
 		"Colorize the output when the output format is one of 'compact' or 'dict'. The value is one of 'auto' (default), 'always' or 'never'",
 	)
-	observeCmd.Flags().AddFlagSet(observeFormattingFlags)
-
-	// generic formatting flags, available to `hubble observe`, including sub-commands.
-	formattingFlags := pflag.NewFlagSet("Formatting", pflag.ContinueOnError)
-	formattingFlags.StringVarP(
-		&formattingOpts.output, "output", "o", "compact",
-		`Specify the output format, one of:
-  compact:  Compact output
-  dict:     Each flow is shown as KEY:VALUE pair
-  json:     JSON encoding (DEPRECATED: use --output=jsonpb instead)
-  jsonpb:   Output each GetFlowResponse according to proto3's JSON mapping
-  table:    Tab-aligned columns
-`)
-	formattingFlags.BoolVarP(&formattingOpts.nodeName, "print-node-name", "", false, "Print node name in output")
-	formattingFlags.StringVar(
-		&formattingOpts.timeFormat, "time-format", "StampMilli",
-		fmt.Sprintf(`Specify the time format for printing. This option does not apply to the json and jsonpb output type. One of:
-  StampMilli:             %s
-  YearMonthDay:           %s
-  YearMonthDayHour:       %s
-  YearMonthDayHourMinute: %s
-  RFC3339:                %s
-  RFC3339Milli:           %s
-  RFC3339Micro:           %s
-  RFC3339Nano:            %s
-  RFC1123Z:               %s
- `,
-			time.StampMilli,
-			hubtime.YearMonthDay,
-			hubtime.YearMonthDayHour,
-			hubtime.YearMonthDayHourMinute,
-			time.RFC3339,
-			hubtime.RFC3339Milli,
-			hubtime.RFC3339Micro,
-			time.RFC3339Nano,
-			time.RFC1123Z,
-		),
-	)
-	observeCmd.PersistentFlags().AddFlagSet(formattingFlags)
-
-	// other flags
-	otherFlags := pflag.NewFlagSet("other", pflag.ContinueOnError)
-	otherFlags.BoolVarP(&otherOpts.ignoreStderr,
-		"silent-errors", "s", false,
-		"Silently ignores errors and warnings")
-	otherFlags.BoolVar(&otherOpts.printRawFilters,
-		"print-raw-filters", false,
-		"Print allowlist/denylist filters and exit without sending the request to Hubble server")
-	observeCmd.PersistentFlags().AddFlagSet(otherFlags)
 
 	// advanced completion for flags
-	observeCmd.RegisterFlagCompletionFunc("ip-version", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("ip-version", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return []string{"none", "v4", "v6"}, cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("type", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("type", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		var completions []string
 		for _, ftype := range flowEventTypes {
 			completions = append(completions, ftype)
@@ -508,13 +465,13 @@ more.`,
 		}
 		return completions, cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("verdict", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("verdict", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return verdicts, cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("protocol", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("protocol", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return protocols, cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("http-status", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("http-status", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		httpStatus := []string{
 			"100", "101", "102", "103",
 			"200", "201", "202", "203", "204", "205", "206", "207", "208",
@@ -530,7 +487,7 @@ more.`,
 		}
 		return httpStatus, cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("http-method", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("http-method", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return []string{
 			http.MethodConnect,
 			http.MethodDelete,
@@ -543,16 +500,19 @@ more.`,
 			http.MethodTrace,
 		}, cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("identity", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("identity", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return reservedIdentitiesNames(), cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("to-identity", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("to-identity", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return reservedIdentitiesNames(), cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("from-identity", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("from-identity", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return reservedIdentitiesNames(), cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("output", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("traffic-direction", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+		return []string{"ingress", "egress"}, cobra.ShellCompDirectiveDefault
+	})
+	flowsCmd.RegisterFlagCompletionFunc("output", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return []string{
 			"compact",
 			"dict",
@@ -561,48 +521,50 @@ more.`,
 			"table",
 		}, cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("color", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("color", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return []string{"auto", "always", "never"}, cobra.ShellCompDirectiveDefault
 	})
-	observeCmd.RegisterFlagCompletionFunc("time-format", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	flowsCmd.RegisterFlagCompletionFunc("time-format", func(_ *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 		return hubtime.FormatNames, cobra.ShellCompDirectiveDefault
 	})
 
+	flagSets := []*pflag.FlagSet{selectorFlags, filterFlags, rawFilterFlags, formattingFlags, flowsFormattingFlags, config.ServerFlags, otherFlags}
+	for _, fs := range flagSets {
+		flowsCmd.Flags().AddFlagSet(fs)
+	}
 	// default value for when the flag is on the command line without any options
-	observeCmd.Flags().Lookup("not").NoOptDefVal = "true"
-
-	observeCmd.AddCommand(
-		newAgentEventsCommand(vp, selectorFlags, formattingFlags, config.ServerFlags, otherFlags),
-		newDebugEventsCommand(vp, selectorFlags, formattingFlags, config.ServerFlags, otherFlags),
-	)
-
-	formattingFlags.AddFlagSet(observeFormattingFlags)
-
-	template.RegisterFlagSets(observeCmd.Name(), selectorFlags, filterFlags, rawFilterFlags, formattingFlags, config.ServerFlags, otherFlags)
-
-	return observeCmd
+	flowsCmd.Flags().Lookup("not").NoOptDefVal = "true"
+	template.RegisterFlagSets(flowsCmd, flagSets...)
+	return flowsCmd
 }
 
-func handleFlowArgs(ofilter *flowFilter, debug bool) (err error) {
+func handleFlowArgs(writer io.Writer, ofilter *flowFilter, debug bool) (err error) {
 	if ofilter.blacklisting {
 		return errors.New("trailing --not found in the arguments")
 	}
 
 	// initialize the printer with any options that were passed in
 	var opts = []hubprinter.Option{
+		hubprinter.Writer(writer),
 		hubprinter.WithTimeFormat(hubtime.FormatNameToLayout(formattingOpts.timeFormat)),
 		hubprinter.WithColor(formattingOpts.color),
 	}
 
+	jsonOut := false
 	switch formattingOpts.output {
 	case "compact":
 		opts = append(opts, hubprinter.Compact())
 	case "dict":
 		opts = append(opts, hubprinter.Dict())
 	case "json", "JSON":
-		opts = append(opts, hubprinter.JSON())
+		if config.Compat.LegacyJSONOutput {
+			opts = append(opts, hubprinter.JSONLegacy())
+			break
+		}
+		fallthrough
 	case "jsonpb":
 		opts = append(opts, hubprinter.JSONPB())
+		jsonOut = true
 	case "tab", "table":
 		if selectorOpts.follow {
 			return fmt.Errorf("table output format is not compatible with follow mode")
@@ -611,6 +573,15 @@ func handleFlowArgs(ofilter *flowFilter, debug bool) (err error) {
 	default:
 		return fmt.Errorf("invalid output format: %s", formattingOpts.output)
 	}
+	if !jsonOut {
+		if len(experimentalOpts.fieldMask) > 0 {
+			return fmt.Errorf("%s output format is not compatible with custom field mask", formattingOpts.output)
+		}
+		if experimentalOpts.useDefaultMasks {
+			experimentalOpts.fieldMask = defaults.FieldMask
+		}
+	}
+
 	if otherOpts.ignoreStderr {
 		opts = append(opts, hubprinter.IgnoreStderr())
 	}
@@ -648,7 +619,7 @@ func parseRawFilters(filters []string) ([]*flowpb.FlowFilter, error) {
 	return results, nil
 }
 
-func getFlowsRequest(ofilter *flowFilter, allowlist []string, denylist []string) (*observer.GetFlowsRequest, error) {
+func getFlowsRequest(ofilter *flowFilter, allowlist []string, denylist []string) (*observerpb.GetFlowsRequest, error) {
 	first := selectorOpts.first > 0
 	last := selectorOpts.last > 0
 	if first && last {
@@ -676,18 +647,18 @@ func getFlowsRequest(ofilter *flowFilter, allowlist []string, denylist []string)
 		if err := since.CheckValid(); err != nil {
 			return nil, fmt.Errorf("failed to convert `since` timestamp to proto: %v", err)
 		}
-		// Set the until field if both --since and --until options are specified and --follow
-		// is not specified. If --since is specified but --until is not, the server sets the
-		// --until option to the current timestamp.
-		if selectorOpts.until != "" && !selectorOpts.follow {
-			ut, err := hubtime.FromString(selectorOpts.until)
-			if err != nil {
-				return nil, fmt.Errorf("failed to parse the until time: %v", err)
-			}
-			until = timestamppb.New(ut)
-			if err := until.CheckValid(); err != nil {
-				return nil, fmt.Errorf("failed to convert `until` timestamp to proto: %v", err)
-			}
+	}
+	// Set the until field if --until option is specified and --follow
+	// is not specified. If --since is specified but --until is not, the server sets the
+	// --until option to the current timestamp.
+	if selectorOpts.until != "" && !selectorOpts.follow {
+		ut, err := hubtime.FromString(selectorOpts.until)
+		if err != nil {
+			return nil, fmt.Errorf("failed to parse the until time: %v", err)
+		}
+		until = timestamppb.New(ut)
+		if err := until.CheckValid(); err != nil {
+			return nil, fmt.Errorf("failed to convert `until` timestamp to proto: %v", err)
 		}
 	}
 
@@ -732,7 +703,7 @@ func getFlowsRequest(ofilter *flowFilter, allowlist []string, denylist []string)
 		number = selectorOpts.first
 	}
 
-	req := &observer.GetFlowsRequest{
+	req := &observerpb.GetFlowsRequest{
 		Number:    number,
 		Follow:    selectorOpts.follow,
 		Whitelist: wl,
@@ -742,10 +713,20 @@ func getFlowsRequest(ofilter *flowFilter, allowlist []string, denylist []string)
 		First:     first,
 	}
 
+	if len(experimentalOpts.fieldMask) > 0 {
+		fm, err := fieldmaskpb.New(&flowpb.Flow{}, experimentalOpts.fieldMask...)
+		if err != nil {
+			return nil, fmt.Errorf("failed to construct field mask: %w", err)
+		}
+		req.Experimental = &observerpb.GetFlowsRequest_Experimental{
+			FieldMask: fm,
+		}
+	}
+
 	return req, nil
 }
 
-func getFlows(ctx context.Context, client observer.ObserverClient, req *observer.GetFlowsRequest) error {
+func getFlows(ctx context.Context, client observerpb.ObserverClient, req *observerpb.GetFlowsRequest) error {
 	b, err := client.GetFlows(ctx, req)
 	if err != nil {
 		return err
