@@ -1,16 +1,5 @@
-// Copyright 2019 Authors of Hubble
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Hubble
 
 package printer
 
@@ -20,62 +9,91 @@ import (
 	"testing"
 	"time"
 
-	pb "github.com/cilium/cilium/api/v1/flow"
+	flowpb "github.com/cilium/cilium/api/v1/flow"
 	observerpb "github.com/cilium/cilium/api/v1/observer"
 	monitorAPI "github.com/cilium/cilium/pkg/monitor/api"
+	"github.com/cilium/hubble/pkg/defaults"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"google.golang.org/protobuf/proto"
+	"google.golang.org/protobuf/reflect/protoreflect"
 	"google.golang.org/protobuf/types/known/timestamppb"
 	"google.golang.org/protobuf/types/known/wrapperspb"
 )
 
-func TestPrinter_WriteProtoFlow(t *testing.T) {
-	buf := bytes.Buffer{}
-	f := pb.Flow{
-		Time: &timestamppb.Timestamp{
-			Seconds: 1234,
-			Nanos:   567800000,
-		},
-		Type:     pb.FlowType_L3_L4,
-		NodeName: "k8s1",
-		Verdict:  pb.Verdict_DROPPED,
-		IP: &pb.IP{
-			Source:      "1.1.1.1",
-			Destination: "2.2.2.2",
-		},
-		Source: &pb.Endpoint{
-			Identity: 4,
-		},
-		Destination: &pb.Endpoint{
-			Identity: 12345,
-		},
-		L4: &pb.Layer4{
-			Protocol: &pb.Layer4_TCP{
-				TCP: &pb.TCP{
-					SourcePort:      31793,
-					DestinationPort: 8080,
-				},
+var f = flowpb.Flow{
+	Time: &timestamppb.Timestamp{
+		Seconds: 1234,
+		Nanos:   567800000,
+	},
+	Type:     flowpb.FlowType_L3_L4,
+	NodeName: "k8s1",
+	Verdict:  flowpb.Verdict_DROPPED,
+	IP: &flowpb.IP{
+		Source:      "1.1.1.1",
+		Destination: "2.2.2.2",
+	},
+	Source: &flowpb.Endpoint{
+		Identity: 4,
+	},
+	Destination: &flowpb.Endpoint{
+		Identity: 12345,
+	},
+	L4: &flowpb.Layer4{
+		Protocol: &flowpb.Layer4_TCP{
+			TCP: &flowpb.TCP{
+				SourcePort:      31793,
+				DestinationPort: 8080,
 			},
 		},
-		EventType: &pb.CiliumEventType{
-			Type:    monitorAPI.MessageTypeDrop,
-			SubType: 133,
-		},
-		Summary: "TCP Flags: SYN",
-		IsReply: &wrapperspb.BoolValue{Value: false},
+	},
+	EventType: &flowpb.CiliumEventType{
+		Type:    monitorAPI.MessageTypeDrop,
+		SubType: 133,
+	},
+	Summary: "TCP Flags: SYN",
+	IsReply: &wrapperspb.BoolValue{Value: false},
+}
+
+func TestPrinter_AllFieldsInMask(t *testing.T) {
+	fm := make(map[string]bool)
+	for _, field := range defaults.FieldMask {
+		fm[field] = true
 	}
-	reply := proto.Clone(&f).(*pb.Flow)
+	check := func(msg protoreflect.Message, prefix string) {
+		fds := msg.Descriptor().Fields()
+		for i := 0; i < fds.Len(); i++ {
+			fd := fds.Get(i)
+			if !msg.Has(fd) {
+				continue
+			}
+			name := prefix + string(fd.Name())
+			if name == "source" || name == "destination" {
+				// Skip compound fields.
+				continue
+			}
+			assert.True(t, fm[name], name)
+		}
+	}
+	check(f.ProtoReflect(), "")
+	check(f.Source.ProtoReflect(), "source.")
+	check(f.Destination.ProtoReflect(), "destination.")
+}
+
+func TestPrinter_WriteProtoFlow(t *testing.T) {
+	buf := bytes.Buffer{}
+	reply := proto.Clone(&f).(*flowpb.Flow)
 	reply.IsReply = &wrapperspb.BoolValue{Value: true}
-	unknown := proto.Clone(&f).(*pb.Flow)
+	unknown := proto.Clone(&f).(*flowpb.Flow)
 	unknown.IsReply = nil
-	policyDenied := proto.Clone(&f).(*pb.Flow)
-	policyDenied.EventType = &pb.CiliumEventType{
+	policyDenied := proto.Clone(&f).(*flowpb.Flow)
+	policyDenied.EventType = &flowpb.CiliumEventType{
 		Type: monitorAPI.MessageTypePolicyVerdict,
 	}
 	policyDenied.IsReply = nil
+	policyDenied.TrafficDirection = flowpb.TrafficDirection_EGRESS
 	type args struct {
-		f *pb.Flow
+		f *flowpb.Flow
 	}
 	tests := []struct {
 		name     string
@@ -172,7 +190,7 @@ Jan  1 00:20:34.567   k8s1   1.1.1.1:31793   2.2.2.2:8080   Policy denied   DROP
 			wantErr: false,
 			expected: "Jan  1 00:20:34.567 [k8s1]: " +
 				"1.1.1.1:31793 (health) <> 2.2.2.2:8080 (ID:12345) " +
-				"policy-verdict:none DENIED (TCP Flags: SYN)\n",
+				"policy-verdict:none EGRESS DENIED (TCP Flags: SYN)\n",
 		},
 		{
 			name: "compact-direction-unknown",
@@ -193,7 +211,7 @@ Jan  1 00:20:34.567   k8s1   1.1.1.1:31793   2.2.2.2:8080   Policy denied   DROP
 		{
 			name: "json",
 			options: []Option{
-				JSON(),
+				JSONPB(),
 				WithColor("never"),
 				Writer(&buf),
 			},
@@ -201,14 +219,14 @@ Jan  1 00:20:34.567   k8s1   1.1.1.1:31793   2.2.2.2:8080   Policy denied   DROP
 				f: &f,
 			},
 			wantErr: false,
-			expected: `{"time":"1970-01-01T00:20:34.567800Z",` +
+			expected: `{"flow":{"time":"1970-01-01T00:20:34.567800Z",` +
 				`"verdict":"DROPPED",` +
 				`"IP":{"source":"1.1.1.1","destination":"2.2.2.2"},` +
 				`"l4":{"TCP":{"source_port":31793,"destination_port":8080}},` +
 				`"source":{"identity":4},"destination":{"identity":12345},` +
 				`"Type":"L3_L4","node_name":"k8s1",` +
 				`"event_type":{"type":1,"sub_type":133},` +
-				`"is_reply":false,"Summary":"TCP Flags: SYN"}`,
+				`"is_reply":false,"Summary":"TCP Flags: SYN"}}`,
 		},
 		{
 			name: "jsonpb",
@@ -276,7 +294,7 @@ DESTINATION: 2.2.2.2:8080
 			res := &observerpb.GetFlowsResponse{
 				ResponseTypes: &observerpb.GetFlowsResponse_Flow{Flow: tt.args.f},
 			}
-			//writes a node status event into the error stream
+			// writes a node status event into the error stream
 			if err := p.WriteProtoFlow(res); (err != nil) != tt.wantErr {
 				t.Errorf("WriteProtoFlow() error = %v, wantErr %v", err, tt.wantErr)
 			}
@@ -288,7 +306,7 @@ DESTINATION: 2.2.2.2:8080
 
 func Test_getHostNames(t *testing.T) {
 	type args struct {
-		f *pb.Flow
+		f *flowpb.Flow
 	}
 	type want struct {
 		src, dst string
@@ -305,14 +323,14 @@ func Test_getHostNames(t *testing.T) {
 		}, {
 			name: "nil ip",
 			args: args{
-				f: &pb.Flow{},
+				f: &flowpb.Flow{},
 			},
 			want: want{},
 		}, {
 			name: "valid ips",
 			args: args{
-				f: &pb.Flow{
-					IP: &pb.IP{
+				f: &flowpb.Flow{
+					IP: &flowpb.IP{
 						Source:      "1.1.1.1",
 						Destination: "2.2.2.2",
 					},
@@ -325,16 +343,16 @@ func Test_getHostNames(t *testing.T) {
 		}, {
 			name: "valid ips/endpoints",
 			args: args{
-				f: &pb.Flow{
-					IP: &pb.IP{
+				f: &flowpb.Flow{
+					IP: &flowpb.IP{
 						Source:      "1.1.1.1",
 						Destination: "2.2.2.2",
 					},
-					Source: &pb.Endpoint{
+					Source: &flowpb.Endpoint{
 						Namespace: "srcns",
 						PodName:   "srcpod",
 					},
-					Destination: &pb.Endpoint{
+					Destination: &flowpb.Endpoint{
 						Namespace: "dstns",
 						PodName:   "dstpod",
 					},
@@ -347,14 +365,14 @@ func Test_getHostNames(t *testing.T) {
 		}, {
 			name: "valid tcp",
 			args: args{
-				f: &pb.Flow{
-					IP: &pb.IP{
+				f: &flowpb.Flow{
+					IP: &flowpb.IP{
 						Source:      "1.1.1.1",
 						Destination: "2.2.2.2",
 					},
-					L4: &pb.Layer4{
-						Protocol: &pb.Layer4_TCP{
-							TCP: &pb.TCP{
+					L4: &flowpb.Layer4{
+						Protocol: &flowpb.Layer4_TCP{
+							TCP: &flowpb.TCP{
 								SourcePort:      55555,
 								DestinationPort: 80,
 							},
@@ -369,14 +387,14 @@ func Test_getHostNames(t *testing.T) {
 		}, {
 			name: "valid udp",
 			args: args{
-				f: &pb.Flow{
-					IP: &pb.IP{
+				f: &flowpb.Flow{
+					IP: &flowpb.IP{
 						Source:      "1.1.1.1",
 						Destination: "2.2.2.2",
 					},
-					L4: &pb.Layer4{
-						Protocol: &pb.Layer4_UDP{
-							UDP: &pb.UDP{
+					L4: &flowpb.Layer4{
+						Protocol: &flowpb.Layer4_UDP{
+							UDP: &flowpb.UDP{
 								SourcePort:      55555,
 								DestinationPort: 53,
 							},
@@ -389,26 +407,48 @@ func Test_getHostNames(t *testing.T) {
 				dst: "2.2.2.2:53",
 			},
 		}, {
-			name: "valid tcp service",
+			name: "valid sctp",
 			args: args{
-				f: &pb.Flow{
-					IP: &pb.IP{
+				f: &flowpb.Flow{
+					IP: &flowpb.IP{
 						Source:      "1.1.1.1",
 						Destination: "2.2.2.2",
 					},
-					L4: &pb.Layer4{
-						Protocol: &pb.Layer4_TCP{
-							TCP: &pb.TCP{
+					L4: &flowpb.Layer4{
+						Protocol: &flowpb.Layer4_SCTP{
+							SCTP: &flowpb.SCTP{
+								SourcePort:      55555,
+								DestinationPort: 5060,
+							},
+						},
+					},
+				},
+			},
+			want: want{
+				src: "1.1.1.1:55555",
+				dst: "2.2.2.2:5060",
+			},
+		}, {
+			name: "valid tcp service",
+			args: args{
+				f: &flowpb.Flow{
+					IP: &flowpb.IP{
+						Source:      "1.1.1.1",
+						Destination: "2.2.2.2",
+					},
+					L4: &flowpb.Layer4{
+						Protocol: &flowpb.Layer4_TCP{
+							TCP: &flowpb.TCP{
 								SourcePort:      55555,
 								DestinationPort: 80,
 							},
 						},
 					},
-					SourceService: &pb.Service{
+					SourceService: &flowpb.Service{
 						Name:      "xwing",
 						Namespace: "default",
 					},
-					DestinationService: &pb.Service{
+					DestinationService: &flowpb.Service{
 						Name:      "tiefighter",
 						Namespace: "deathstar",
 					},
@@ -421,24 +461,24 @@ func Test_getHostNames(t *testing.T) {
 		}, {
 			name: "valid udp service",
 			args: args{
-				f: &pb.Flow{
-					IP: &pb.IP{
+				f: &flowpb.Flow{
+					IP: &flowpb.IP{
 						Source:      "1.1.1.1",
 						Destination: "2.2.2.2",
 					},
-					L4: &pb.Layer4{
-						Protocol: &pb.Layer4_UDP{
-							UDP: &pb.UDP{
+					L4: &flowpb.Layer4{
+						Protocol: &flowpb.Layer4_UDP{
+							UDP: &flowpb.UDP{
 								SourcePort:      55555,
 								DestinationPort: 53,
 							},
 						},
 					},
-					SourceService: &pb.Service{
+					SourceService: &flowpb.Service{
 						Name:      "xwing",
 						Namespace: "default",
 					},
-					DestinationService: &pb.Service{
+					DestinationService: &flowpb.Service{
 						Name:      "tiefighter",
 						Namespace: "deathstar",
 					},
@@ -451,14 +491,14 @@ func Test_getHostNames(t *testing.T) {
 		}, {
 			name: "dns",
 			args: args{
-				f: &pb.Flow{
-					IP: &pb.IP{
+				f: &flowpb.Flow{
+					IP: &flowpb.IP{
 						Source:      "1.1.1.1",
 						Destination: "2.2.2.2",
 					},
-					L4: &pb.Layer4{
-						Protocol: &pb.Layer4_TCP{
-							TCP: &pb.TCP{
+					L4: &flowpb.Layer4{
+						Protocol: &flowpb.Layer4_TCP{
+							TCP: &flowpb.TCP{
 								SourcePort:      54321,
 								DestinationPort: 65432,
 							},
@@ -476,8 +516,8 @@ func Test_getHostNames(t *testing.T) {
 		{
 			name: "ethernet",
 			args: args{
-				f: &pb.Flow{
-					Ethernet: &pb.Ethernet{
+				f: &flowpb.Flow{
+					Ethernet: &flowpb.Ethernet{
 						Source:      "00:01:02:03:04:05",
 						Destination: "06:07:08:09:0a:0b",
 					},
@@ -563,7 +603,7 @@ func Test_fmtTimestamp(t *testing.T) {
 
 func Test_getFlowType(t *testing.T) {
 	type args struct {
-		f *pb.Flow
+		f *flowpb.Flow
 	}
 	tests := []struct {
 		name string
@@ -573,11 +613,11 @@ func Test_getFlowType(t *testing.T) {
 		{
 			name: "L7",
 			args: args{
-				f: &pb.Flow{
-					L7: &pb.Layer7{
-						Type: pb.L7FlowType_REQUEST,
+				f: &flowpb.Flow{
+					L7: &flowpb.Layer7{
+						Type: flowpb.L7FlowType_REQUEST,
 					},
-					EventType: &pb.CiliumEventType{
+					EventType: &flowpb.CiliumEventType{
 						Type: monitorAPI.MessageTypeAccessLog,
 					},
 				},
@@ -587,12 +627,12 @@ func Test_getFlowType(t *testing.T) {
 		{
 			name: "HTTP",
 			args: args{
-				f: &pb.Flow{
-					L7: &pb.Layer7{
-						Type:   pb.L7FlowType_RESPONSE,
-						Record: &pb.Layer7_Http{},
+				f: &flowpb.Flow{
+					L7: &flowpb.Layer7{
+						Type:   flowpb.L7FlowType_RESPONSE,
+						Record: &flowpb.Layer7_Http{},
 					},
-					EventType: &pb.CiliumEventType{
+					EventType: &flowpb.CiliumEventType{
 						Type: monitorAPI.MessageTypeAccessLog,
 					},
 				},
@@ -602,12 +642,12 @@ func Test_getFlowType(t *testing.T) {
 		{
 			name: "Kafka",
 			args: args{
-				f: &pb.Flow{
-					L7: &pb.Layer7{
-						Type:   pb.L7FlowType_REQUEST,
-						Record: &pb.Layer7_Kafka{},
+				f: &flowpb.Flow{
+					L7: &flowpb.Layer7{
+						Type:   flowpb.L7FlowType_REQUEST,
+						Record: &flowpb.Layer7_Kafka{},
 					},
-					EventType: &pb.CiliumEventType{
+					EventType: &flowpb.CiliumEventType{
 						Type: monitorAPI.MessageTypeAccessLog,
 					},
 				},
@@ -617,23 +657,25 @@ func Test_getFlowType(t *testing.T) {
 		{
 			name: "DNS",
 			args: args{
-				f: &pb.Flow{
-					L7: &pb.Layer7{
-						Type:   pb.L7FlowType_REQUEST,
-						Record: &pb.Layer7_Dns{},
+				f: &flowpb.Flow{
+					L7: &flowpb.Layer7{
+						Type: flowpb.L7FlowType_REQUEST,
+						Record: &flowpb.Layer7_Dns{
+							Dns: &flowpb.DNS{ObservationSource: "proxy"},
+						},
 					},
-					EventType: &pb.CiliumEventType{
+					EventType: &flowpb.CiliumEventType{
 						Type: monitorAPI.MessageTypeAccessLog,
 					},
 				},
 			},
-			want: "dns-request",
+			want: "dns-request proxy",
 		},
 		{
 			name: "L4",
 			args: args{
-				f: &pb.Flow{
-					EventType: &pb.CiliumEventType{
+				f: &flowpb.Flow{
+					EventType: &flowpb.CiliumEventType{
 						Type:    monitorAPI.MessageTypeTrace,
 						SubType: monitorAPI.TraceToHost,
 					},
@@ -644,37 +686,65 @@ func Test_getFlowType(t *testing.T) {
 		{
 			name: "L4",
 			args: args{
-				f: &pb.Flow{
-					Verdict: pb.Verdict_FORWARDED,
-					EventType: &pb.CiliumEventType{
+				f: &flowpb.Flow{
+					Verdict: flowpb.Verdict_FORWARDED,
+					EventType: &flowpb.CiliumEventType{
 						Type: monitorAPI.MessageTypePolicyVerdict,
 					},
-					PolicyMatchType: monitorAPI.PolicyMatchL3L4,
+					PolicyMatchType:  monitorAPI.PolicyMatchL3L4,
+					TrafficDirection: flowpb.TrafficDirection_INGRESS,
 				},
 			},
-			want: "policy-verdict:L3-L4",
+			want: "policy-verdict:L3-L4 INGRESS",
 		},
 		{
 			name: "L4",
 			args: args{
-				f: &pb.Flow{
-					Verdict: pb.Verdict_DROPPED,
-					EventType: &pb.CiliumEventType{
+				f: &flowpb.Flow{
+					Verdict: flowpb.Verdict_DROPPED,
+					EventType: &flowpb.CiliumEventType{
 						Type: monitorAPI.MessageTypePolicyVerdict,
 					},
-					DropReason: 153,
+					DropReason:       153,
+					TrafficDirection: flowpb.TrafficDirection_INGRESS,
 				},
 			},
-			want: "policy-verdict:none",
+			want: "policy-verdict:none INGRESS",
+		},
+		{
+			name: "SockLB pre-translate",
+			args: args{
+				f: &flowpb.Flow{
+					Verdict: flowpb.Verdict_TRACED,
+					EventType: &flowpb.CiliumEventType{
+						Type: monitorAPI.MessageTypeTraceSock,
+					},
+					SockXlatePoint: flowpb.SocketTranslationPoint_SOCK_XLATE_POINT_PRE_DIRECTION_FWD,
+				},
+			},
+			want: "pre-xlate-fwd",
+		},
+		{
+			name: "SockLB post-translate",
+			args: args{
+				f: &flowpb.Flow{
+					Verdict: flowpb.Verdict_TRANSLATED,
+					EventType: &flowpb.CiliumEventType{
+						Type: monitorAPI.MessageTypeTraceSock,
+					},
+					SockXlatePoint: flowpb.SocketTranslationPoint_SOCK_XLATE_POINT_POST_DIRECTION_FWD,
+				},
+			},
+			want: "post-xlate-fwd",
 		},
 		{
 			name: "Debug Capture",
 			args: args{
-				f: &pb.Flow{
-					EventType: &pb.CiliumEventType{
+				f: &flowpb.Flow{
+					EventType: &flowpb.CiliumEventType{
 						Type: monitorAPI.MessageTypeCapture,
 					},
-					DebugCapturePoint: pb.DebugCapturePoint_DBG_CAPTURE_FROM_LB,
+					DebugCapturePoint: flowpb.DebugCapturePoint_DBG_CAPTURE_FROM_LB,
 				},
 			},
 			want: "DBG_CAPTURE_FROM_LB",
@@ -682,8 +752,8 @@ func Test_getFlowType(t *testing.T) {
 		{
 			name: "invalid",
 			args: args{
-				f: &pb.Flow{
-					EventType: &pb.CiliumEventType{
+				f: &flowpb.Flow{
+					EventType: &flowpb.CiliumEventType{
 						Type:    monitorAPI.MessageTypeTrace,
 						SubType: 123, // invalid subtype
 					},
@@ -700,7 +770,7 @@ func Test_getFlowType(t *testing.T) {
 		{
 			name: "nil type",
 			args: args{
-				f: &pb.Flow{},
+				f: &flowpb.Flow{},
 			},
 			want: "UNKNOWN",
 		},
@@ -733,7 +803,7 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 
 	tests := []struct {
 		name string
-		ev   *pb.AgentEvent
+		ev   *flowpb.AgentEvent
 		want string
 	}{
 		{
@@ -742,29 +812,29 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "empty",
-			ev:   &pb.AgentEvent{},
+			ev:   &flowpb.AgentEvent{},
 			want: "UNKNOWN",
 		},
 		{
 			name: "unknown without notification",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_AGENT_EVENT_UNKNOWN,
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_AGENT_EVENT_UNKNOWN,
 			},
 			want: "UNKNOWN",
 		},
 		{
 			name: "agent start without notification",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_AGENT_STARTED,
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_AGENT_STARTED,
 			},
 			want: "UNKNOWN",
 		},
 		{
 			name: "agent start with notification",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_AGENT_STARTED,
-				Notification: &pb.AgentEvent_AgentStart{
-					AgentStart: &pb.TimeNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_AGENT_STARTED,
+				Notification: &flowpb.AgentEvent_AgentStart{
+					AgentStart: &flowpb.TimeNotification{
 						Time: startTS,
 					},
 				},
@@ -773,10 +843,10 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "policy update",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_POLICY_UPDATED,
-				Notification: &pb.AgentEvent_PolicyUpdate{
-					PolicyUpdate: &pb.PolicyUpdateNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_POLICY_UPDATED,
+				Notification: &flowpb.AgentEvent_PolicyUpdate{
+					PolicyUpdate: &flowpb.PolicyUpdateNotification{
 						Labels:    []string{"foo=bar", "baz=foo"},
 						Revision:  1,
 						RuleCount: 2,
@@ -787,10 +857,10 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "policy delete",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_POLICY_DELETED,
-				Notification: &pb.AgentEvent_PolicyUpdate{
-					PolicyUpdate: &pb.PolicyUpdateNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_POLICY_DELETED,
+				Notification: &flowpb.AgentEvent_PolicyUpdate{
+					PolicyUpdate: &flowpb.PolicyUpdateNotification{
 						Revision:  42,
 						RuleCount: 1,
 					},
@@ -800,10 +870,10 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "endpoint regenerate success",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_ENDPOINT_REGENERATE_SUCCESS,
-				Notification: &pb.AgentEvent_EndpointRegenerate{
-					EndpointRegenerate: &pb.EndpointRegenNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_ENDPOINT_REGENERATE_SUCCESS,
+				Notification: &flowpb.AgentEvent_EndpointRegenerate{
+					EndpointRegenerate: &flowpb.EndpointRegenNotification{
 						Id:     42,
 						Labels: []string{"baz=bar", "some=label"},
 					},
@@ -813,10 +883,10 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "endpoint regenerate failure",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_ENDPOINT_REGENERATE_FAILURE,
-				Notification: &pb.AgentEvent_EndpointRegenerate{
-					EndpointRegenerate: &pb.EndpointRegenNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_ENDPOINT_REGENERATE_FAILURE,
+				Notification: &flowpb.AgentEvent_EndpointRegenerate{
+					EndpointRegenerate: &flowpb.EndpointRegenNotification{
 						Id:     42,
 						Labels: []string{"baz=bar", "some=label"},
 						Error:  "some error",
@@ -827,10 +897,10 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "endpoint created",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_ENDPOINT_CREATED,
-				Notification: &pb.AgentEvent_EndpointUpdate{
-					EndpointUpdate: &pb.EndpointUpdateNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_ENDPOINT_CREATED,
+				Notification: &flowpb.AgentEvent_EndpointUpdate{
+					EndpointUpdate: &flowpb.EndpointUpdateNotification{
 						Id:        1027,
 						Namespace: "kube-system",
 						PodName:   "cilium-xyz",
@@ -841,10 +911,10 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "ipcache upsert",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_IPCACHE_UPSERTED,
-				Notification: &pb.AgentEvent_IpcacheUpdate{
-					IpcacheUpdate: &pb.IPCacheNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_IPCACHE_UPSERTED,
+				Notification: &flowpb.AgentEvent_IpcacheUpdate{
+					IpcacheUpdate: &flowpb.IPCacheNotification{
 						Cidr:     "10.1.2.3/32",
 						Identity: 42,
 						OldIdentity: &wrapperspb.UInt32Value{
@@ -859,10 +929,10 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "ipcache delete",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_IPCACHE_DELETED,
-				Notification: &pb.AgentEvent_IpcacheUpdate{
-					IpcacheUpdate: &pb.IPCacheNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_IPCACHE_DELETED,
+				Notification: &flowpb.AgentEvent_IpcacheUpdate{
+					IpcacheUpdate: &flowpb.IPCacheNotification{
 						Cidr:      "10.0.1.2/32",
 						Identity:  42,
 						OldHostIp: "192.168.1.23",
@@ -873,16 +943,16 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "service upsert",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_SERVICE_UPSERTED,
-				Notification: &pb.AgentEvent_ServiceUpsert{
-					ServiceUpsert: &pb.ServiceUpsertNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_SERVICE_UPSERTED,
+				Notification: &flowpb.AgentEvent_ServiceUpsert{
+					ServiceUpsert: &flowpb.ServiceUpsertNotification{
 						Id: 42,
-						FrontendAddress: &pb.ServiceUpsertNotificationAddr{
+						FrontendAddress: &flowpb.ServiceUpsertNotificationAddr{
 							Ip:   "10.0.0.42",
 							Port: 8008,
 						},
-						BackendAddresses: []*pb.ServiceUpsertNotificationAddr{
+						BackendAddresses: []*flowpb.ServiceUpsertNotificationAddr{
 							{
 								Ip:   "192.168.1.23",
 								Port: 80,
@@ -903,10 +973,10 @@ func TestPrinter_AgentEventDetails(t *testing.T) {
 		},
 		{
 			name: "service delete",
-			ev: &pb.AgentEvent{
-				Type: pb.AgentEventType_SERVICE_DELETED,
-				Notification: &pb.AgentEvent_ServiceDelete{
-					ServiceDelete: &pb.ServiceDeleteNotification{
+			ev: &flowpb.AgentEvent{
+				Type: flowpb.AgentEventType_SERVICE_DELETED,
+				Notification: &flowpb.AgentEvent_ServiceDelete{
+					ServiceDelete: &flowpb.ServiceDeleteNotification{
 						Id: 42,
 					},
 				},
@@ -931,9 +1001,9 @@ func TestPrinter_WriteProtoDebugEvent(t *testing.T) {
 		Nanos:   567800000,
 	}
 	node := "k8s1"
-	dbg := &pb.DebugEvent{
-		Type: pb.DebugEventType_DBG_CT_VERDICT,
-		Source: &pb.Endpoint{
+	dbg := &flowpb.DebugEvent{
+		Type: flowpb.DebugEventType_DBG_CT_VERDICT,
+		Source: &flowpb.Endpoint{
 			ID:        690,
 			Identity:  1332,
 			Namespace: "cilium-test",
@@ -953,7 +1023,7 @@ func TestPrinter_WriteProtoDebugEvent(t *testing.T) {
 		Cpu:     wrapperspb.Int32(1),
 	}
 	type args struct {
-		dbg  *pb.DebugEvent
+		dbg  *flowpb.DebugEvent
 		ts   *timestamppb.Timestamp
 		node string
 	}
@@ -1025,7 +1095,7 @@ Jan  1 00:20:34.567   k8s1   cilium-test/pod-to-a-denied-cnp-75cb89dfd-vqhd9 (ID
 		{
 			name: "json",
 			options: []Option{
-				JSON(),
+				JSONPB(),
 				Writer(&buf),
 			},
 			args: args{
@@ -1034,11 +1104,12 @@ Jan  1 00:20:34.567   k8s1   cilium-test/pod-to-a-denied-cnp-75cb89dfd-vqhd9 (ID
 				ts:   ts,
 			},
 			wantErr: false,
-			expected: `{"type":"DBG_CT_VERDICT","source":{"ID":690,"identity":1332,"namespace":"cilium-test","labels":` +
+			expected: `{"debug_event":{"type":"DBG_CT_VERDICT","source":{"ID":690,"identity":1332,"namespace":"cilium-test","labels":` +
 				`["k8s:io.cilium.k8s.policy.cluster=default","k8s:io.cilium.k8s.policy.serviceaccount=default",` +
 				`"k8s:io.kubernetes.pod.namespace=cilium-test","k8s:name=pod-to-a-denied-cnp"],` +
 				`"pod_name":"pod-to-a-denied-cnp-75cb89dfd-vqhd9"},` +
-				`"hash":180354257,"arg1":0,"arg2":0,"arg3":0,"message":"CT verdict: New, revnat=0","cpu":1}`,
+				`"hash":180354257,"arg1":0,"arg2":0,"arg3":0,"message":"CT verdict: New, revnat=0","cpu":1},` +
+				`"node_name":"k8s1","time":"1970-01-01T00:20:34.567800Z"}`,
 		},
 		{
 			name: "jsonpb",
@@ -1198,7 +1269,7 @@ Connected Nodes: 2/2`,
 		}, {
 			name: "json",
 			options: []Option{
-				JSON(),
+				JSONPB(),
 				WithColor("never"),
 				Writer(&buf),
 			},
@@ -1208,7 +1279,7 @@ Connected Nodes: 2/2`,
 		}, {
 			name: "json-with-nodes",
 			options: []Option{
-				JSON(),
+				JSONPB(),
 				WithColor("never"),
 				Writer(&buf),
 			},
